unit qrcb_run;

{$I versions.inc}

interface

uses graphics, classes, DB, DBTables, quickrpt, BdeConst;

type

TcTickStyle = (tsCross, tsTick);

type
  TQRCustomCheckBox = class(TQRPrintable)
  private
    FTickStyle: TcTickStyle;
    procedure SetTickStyle(Style: TcTickStyle);
  protected
    FChecked : boolean;
    procedure ReadVisible(Reader : TReader); virtual;
    procedure WriteDummy(Writer : TWriter); virtual;
  public
    constructor Create(AOwner : TComponent); override;
    procedure Paint; override;
    procedure Print(OfsX, OfsY : integer); override;
  published
    property TickStyle: TcTickStyle read FTickStyle write SetTickStyle;
  end;

  TQRCheckBox = class(TQRCustomCheckBox)
  public
    constructor Create(AOwner : TComponent); override;
    procedure SetChecked(Value : boolean);
  published
    property Checked : boolean read FChecked write SetChecked;
  end;

  TQRDBCheckBox = class(TQRCustomCheckBox)
  private
    Field : TField;
    FieldNo : integer;
    FieldOK : boolean;
    DataSourceName : string[30];
    FDataSet : TDataSet;
    FDataField : string;
    procedure SetDataSet(Value : TDataSet);
    procedure SetDataField(Value : string);
  protected
    procedure DefineProperties(Filer: TFiler); override;
    procedure Prepare; override;
    procedure ReadValues(Reader : TReader); virtual;
    procedure UnPrepare; override;
    procedure WriteValues(Writer : TWriter); virtual;
  public
    constructor Create(AOwner : TComponent); override;
    procedure Print(OfsX, OfsY : integer); override;
  published
    property DataSet : TDataSet read FDataSet write SetDataSet;
    property DataField : string read FDataField write SetDataField;
    property Checked : boolean read FChecked;
  end;

implementation

constructor TQRCustomCheckBox.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);

  {override default frame settings to suite a checkbox}
  Frame.DrawBottom := True;
  Frame.DrawLeft   := True;
  Frame.DrawRight  := True;
  Frame.DrawTop    := True;
  AutoSize         := False;
end;

procedure TQRCustomCheckBox.SetTickStyle(Style: TcTickStyle);
begin
  if Style in [tsCross, tsTick] then
    if Style <> FTickStyle then
      FTickStyle := Style;
  Invalidate;
end;

procedure TQRCustomCheckBox.Paint;
begin
  with Canvas do
  begin
    Pen.Color := Frame.Color;
    Pen.Width := Frame.Width;
    Pen.Style := Frame.Style;
    MoveTo(0,0);

    if Frame.DrawTop then LineTo(Width-1,0)
	 else MoveTo(Width-1,0);
    if Frame.DrawRight then LineTo(Width-1,Height-1)
	 else MoveTo(Width-1,Height-1);
    if Frame.DrawBottom then LineTo(0,Height-1)
      else MoveTo(0,Height-1);
    if Frame.DrawLeft then LineTo(0,0);

    if FChecked then
    begin
      Case TickStyle of
	tsCross:
	begin
	  MoveTo(0, 0);
	  LineTo(Width, Height - 1);
	  MoveTo(0, Height - 1);
	  LineTo(Width - 1, 0);
	end;

	tsTick:
	begin
	  Pen.Width := Frame.Width + 1;
	  MoveTo(2, 2*(Height div 3));
	  LineTo(Width div 3, Height - 3);
	  LineTo(Width - 2, 2);
	end;
      end;
    end;
  end;
end;

procedure TQRCustomCheckBox.Print(OfsX, OfsY : integer);
var
  CalcLeft, CalcTop, CalcRight, CalcBottom : Longint;
begin
  with ParentReport.QRPrinter do
  begin
    Canvas.Pen.Color := Frame.Color;
    Canvas.Pen.Width := Frame.Width;
    Canvas.Pen.Style := Frame.Style;
    CalcLeft := XPos(OfsX + Size.Left)+1;
    CalcTop := YPos(OfsY + Size.Top)+1;
    CalcRight := XPos(OfsX + Size.Left + Size.Width)-1;
    CalcBottom := YPos(OfsY + Size.Top + Size.Height)-1;

    Canvas.MoveTo(CalcLeft,CalcTop);
    if Frame.DrawTop then Canvas.LineTo(CalcRight,CalcTop)
	 else Canvas.MoveTo(CalcRight,CalcTop);
    if Frame.DrawRight then Canvas.LineTo(CalcRight,CalcBottom)
	 else Canvas.MoveTo(CalcRight,CalcBottom);
    if Frame.DrawBottom then Canvas.LineTo(CalcLeft,CalcBottom)
	 else Canvas.MoveTo(CalcLeft,CalcBottom);
    if Frame.DrawLeft then Canvas.LineTo(CalcLeft,CalcTop);

    if FChecked then
    begin
      Case TickStyle of
	tsCross:
	begin
	  Canvas.MoveTo(CalcLeft, CalcTop);
	  Canvas.LineTo(CalcRight, CalcBottom);
	  Canvas.MoveTo(CalcLeft, CalcBottom);
	  Canvas.LineTo(CalcRight, CalcTop);
	end;

	tsTick:
	begin
	  Canvas.Pen.Width := Frame.Width + 1;
	  Canvas.MoveTo(CalcLeft + 2, CalcTop + 2*((CalcBottom-CalcTop) div 3));
	  Canvas.LineTo(CalcLeft + (CalcRight-CalcLeft) div 3, CalcBottom - 2);
	  Canvas.LineTo(CalcRight - 2, CalcTop + 2);
	end;
      end;
    end;
  end;
end;

procedure TQRCustomCheckBox.ReadVisible(Reader : TReader);
begin
  Enabled := Reader.ReadBoolean;
end;

procedure TQRCustomCheckBox.WriteDummy(Writer : TWriter);
begin
end;

constructor TQRCheckBox.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  FChecked := False;
end;

procedure TQRCheckBox.SetChecked(Value : boolean);
begin
  if Value <> FChecked then
  begin
    FChecked := Value;
    Invalidate;
  end;
end;

constructor TQRDBCheckBox.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  FChecked := FALSE;
  DataSourceName := '';
end;

procedure TQRDBCheckBox.DefineProperties(Filer: TFiler);
begin
  Filer.DefineProperty('DataSource',ReadValues,WriteValues,false);
  Filer.DefineProperty('Visible', ReadVisible, WriteDummy, false);
  inherited DefineProperties(Filer);
end;

procedure TQRDBCheckBox.SetDataSet(Value : TDataSet);
begin
  FDataSet := Value;

  if Value <> nil then
    Value.FreeNotification(self);
end;

procedure TQRDBCheckBox.SetDataField(Value : string);
begin
  FDataField := Value;
end;

procedure TQRDBCheckBox.Prepare;
begin
  inherited Prepare;
  if assigned(FDataSet) then
  begin
    Field := FDataSet.FindField(FDataField);
    if (Field <> nil) and ((Field is TBooleanField)
                       or  (Field is TNumericField) ) Then
    begin
      FieldNo := Field.Index;
      FieldOK := true;
    end else
    begin
      Field := nil;
      FieldOK := false;
    end;
  end else
  begin
    Field := nil;
    FieldOK := false;
  end;
end;

procedure TQRDBCheckBox.UnPrepare;
begin
  Field := nil;
  inherited Unprepare;
  if DataField <> '' then
    SetDataField(DataField) { Reset component caption }
  else
    SetDataField(Name);
end;

procedure TQRDBCheckBox.ReadValues(Reader : TReader);
begin
  DataSourceName := Reader.ReadIdent;
end;

procedure TQRDBCheckBox.WriteValues(Writer : TWriter);
begin
end;

procedure TQRDBCheckBox.Print(OfsX, OfsY : integer);
begin
  if FieldOK then
  begin
    if FDataSet.DefaultFields then
      Field := FDataSet.Fields[FieldNo];
  end
  else
    Field := nil;

  FChecked := FALSE;
  if assigned(Field) then
    if (Field is TBooleanField) then
      FChecked := TBooleanField(Field).value
    else if (Field is TNumericField) then
      FChecked := not (Field.AsInteger = 0);

  inherited Print(OfsX,OfsY);
end;

end.
